/******************************************************************************
 * program:     rasimg library                                                *
 * function:    Executable for image conversion.                              *
 * modul:       convert.cc                                                    *
 * licency:     GPL or LGPL                                                   *
 ******************************************************************************/
#include <stddef.h>
#include <stdio.h>
//#include <io.h>

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "types.h"
#include "raster.h"
#include "imageop.h"

#ifndef __UNIX__
 //#include <io.h>
#endif

#include <matrix.h>

//#ifdef RasImg

//#include <alloc.h>

#ifndef __Have_stricmp
 #ifdef __Have_strcasecmp
  #define stricmp strcasecmp
  #define strnicmp strncasecmp
 #endif
 #ifndef stricmp        //already fixed?
  #define stricmp strcmp
 #endif
#endif


extern Image LoadPicture(const char *Name);
extern int SavePicture(const char *Name,const Image &Img);



const char *GetExtension(const char *FullName)
{
const char *NameStep;
int LastPos;

if(FullName==NULL) return(NULL);

LastPos=strlen(FullName);
NameStep=FullName+LastPos+1;
while(NameStep>FullName)
	{
	NameStep--;

	if(*NameStep=='/' || *NameStep=='\\' || *NameStep==':')
		break;
	if(*NameStep=='.')
		return(NameStep);
	}

return(FullName+LastPos);
}


/////////////////////Main CODE starts here/////////////////////////////////

char Verbosity = 0;

void PrintHeader(void)
{
char HdrPrinted=0;
  if(Verbosity>=0 && HdrPrinted==0)
    {
    puts("\n\r <<< CONVERT >>> Convert picture format (c)1997-2018 F&TSoft ");
    HdrPrinted=1;
    }
}


int main(int argc,char *argv[])
{
Image I1,I2;
const char *InputImg=NULL, *OutputImg=NULL;
const char *s;
const char *OutFormat=NULL;
TImageFileHandler *LoadPicture_XXX;
int i;
int PeelPlane=-1;
int Error = 0;

	/* Parse Arguments */
for(i=1;i<argc;i++)
  {
  s=argv[i];
  if(s==NULL) continue;
  if(*s==0) continue;

  if(*s=='-' || *s=='/')
    {
    if(strcmp(s+1,"help")==0 || strcmp(s+1,"-help")==0 || strcmp(s+1,"?")==0)
      {
        PrintHeader();
        printf("\n");
        for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
	  {
	  printf("%s%s%s %s%s; ",LoadPicture_XXX->extension(),
			LoadPicture_XXX->description()==NULL?"":"#",
			LoadPicture_XXX->description()==NULL?"":LoadPicture_XXX->description(),
			LoadPicture_XXX->LoadPicture==NULL?"":"R",
			LoadPicture_XXX->SavePicture==NULL?"":"W");
	  }
	printf(//"\n/NoScale      - means truncating image data"
	       //"\n(not working yet) /Gray         - discards all color informations"
	       //"\n/SEPARATE[R|G|B] - remove one channel from true color images"
	       "\n/Format \"str\" - specify output format"
	       "\n/PeelPlane n  - cut only plane with number n"
	       "\n/s            - decrease verbosity level"
	       "\n/v            - increase verbosity level"
	       "\n");
        return(0);
      }
    if(strcmp(s+1,"Format")==0 || strcmp(s+1,"format")==0)
      {
      OutFormat = argv[++i];
      continue;
      }
    if(strcmp(s+1,"PeelPlane") == 0)
      {
      PeelPlane = atoi(argv[++i]);
      continue;
      }
    if(strcmp(s+1,"s") == 0)
      {
      Verbosity = -1;
      continue;
      }
    if(strcmp(s+1,"v") == 0)
      {
      Verbosity = +1;
      continue;
      }
    }

  if(InputImg==NULL)
     InputImg=s;
  else
     if(OutputImg==NULL) OutputImg=s;
  }
	/* End of reading arguments */

  PrintHeader();
  if(Verbosity>=0) printf("\rLoading");
  //MarkTime=GetInterval();

//for(i=1;i<=10;i++)
  I1 = LoadPicture(InputImg);

  if(I1.Raster==NULL && I1.Comment==NULL)
	{
	printf("\n\rCannot load input picture:\"%s\"",InputImg);
	goto End;
	}

  if(Verbosity>=0) printf("\rSaving ");
  //MarkTime=GetInterval();

  if(PeelPlane>0)
    {
    I1 = PeelPlane_(I1, PeelPlane);
    }

  if(OutFormat==NULL)
    Error = SavePicture(OutputImg,I1);  //No format requested
  else
    {
    TImageFileHandler *SavePicture_XXX;
    const char *ext;

    Error = 0xFFFF;
  
    for(SavePicture_XXX=TImageFileHandler::first();SavePicture_XXX!=NULL;SavePicture_XXX=SavePicture_XXX->next())
      {
      if(SavePicture_XXX->SavePicture!=NULL)
        {
        ext = SavePicture_XXX->extension();

        if(ext!=NULL)
          {
          if(*ext=='.') ext++;
	  i = strlen(ext);
          if(!stricmp(OutFormat,ext))
            {
	    Error = SavePicture_XXX->SavePicture(OutputImg,I1);	  //Exact match
            break;
	    }
	  if(!strnicmp(OutFormat,ext,i))
            {
	    if(OutFormat[i]=='#')
	      {
	      if(!stricmp(OutFormat+i+1,SavePicture_XXX->description()))
		{
		Error = SavePicture_XXX->SavePicture(OutputImg,I1);  //Description match
		break;
		}	   
	      }
	    }
          }
        }
      }
    }

  if(Error)
    {
    if(OutFormat && Error==0xFFFF)
      printf("\n\rNo export format:\"%s\" found.",OutFormat);
    else
      printf("\n\rCannot save output picture:\"%s\" Error %d",OutputImg,Error);
    }
  else
    {
    if(Verbosity>=0)
      {
      //printf("time : %f",abs(GetInterval()-MarkTime)/18.2);
      puts("\rOK     ");
      }
    }

End:
  return(Error);

}

//#endif //Ras_img

